function [yf opt_fau] = fault_generation(x,y,opt_output,opt_fau)
% FAULT_GENERATION generate faulty signal
%   This function generates a vector YF of the faulty signal from the 
%   healty signal Y and a fault specification OPT_FAU. It stores some
%   information on the faults in OPT_FAU.
%
% Input:
%   opt_output.hor: lenght of the signal (or length(y))
%   opt_output.range: range for the y signal
%   opt_fau.not_fault: number of not faulty data, i.e. time instant for
%       the start of the fault
%   opt_fau.final_intensity_rate: vector of fault intensities
%   opt_fau.stru: flag for the use of structured faults or not
%   opt_fau.fault_type: flag vector for the faults
%   if opt_fau.stru == 0
%       'none' No fault
%       'abr_add' Additive constant fault
%       'dri_add' Additive linearly increasing fault
%       'exp_add' Additive exonantially increasing fault
%       'abr_mul' Multplicative constant fault
%       'dri_mul' Multplicative linearly increasing fault
%       'exp_mul' Multplicative exponentially increasing fault
%       'noise' structured noise
%           1) noise with predefined variance -> define opt_fau.noise_magnitude.sigma_noise
%           2) noise with fixed snr in db (signal to noise ratio) -> define opt_fau.noise_magnitude.snr
%       'stuck_at' Stuck at 
%   if opt_fau.stru == 1
%       1 Fault
%       0 No fault
%   ex. opt_fau.fault_type = [1 0 2] simulate an additive constant fault, 
%       then returns to the nominal condition, then a linerly incresing
%       fault shows           
%  Output:
%     yf: output faulty data
%     opt_fau.n_faults: number of faults
%     opt_fau.per_fault_sample: number of samples for each fault

% Counting faults
opt_fau.n_faults = length(opt_fau.fault_type);
if opt_fau.n_faults ~= 0
    opt_fau.per_fault_sample = floor((opt_output.hor - opt_fau.not_fault) / opt_fau.n_faults);
else
    opt_fau.per_fault_sample = 0;
end

%opt_fau.fau_seq = ones(1,opt_fau.n_faults+1);
opt_fau.fau_times = [0 opt_fau.not_fault opt_fau.per_fault_sample*ones(1,opt_fau.n_faults)];
opt_fau.fau_times = cumsum(opt_fau.fau_times);

% Final value of the fault
range = max(y(1+opt_output.na:end)) - min(y(1+opt_output.na:end));
scale = range * opt_fau.final_intensity_rate;
vari_sig = var(y(1+opt_output.na:end));

% Faulty signal initialization
yf = y;

for i = 1:opt_fau.n_faults
    if ~strcmp(opt_fau.fault_type{i},'none')
        if opt_fau.final_intensity_rate(i) ~= 0
            init_index = opt_fau.fau_times(i+1) + 1;
            final_index = opt_fau.fau_times(i+2);
            if opt_fau.stru(i) == 0
                switch opt_fau.fault_type{i}
                    case 'abr_add' % Additive constant fault
                        fault = y(init_index:final_index) + scale(i) * ones(1,opt_fau.per_fault_sample);
                    case 'dri_add' % Additive linearly increasing fault
                        fault = y(init_index:final_index) + (1:opt_fau.per_fault_sample) / opt_fau.per_fault_sample * scale(i);
                    case 'exp_add' % Additive exonantially increasing fault
                        fault = y(init_index:final_index) + (exp( (0:(opt_fau.per_fault_sample-1)) / (opt_fau.per_fault_sample-1)) - 1) / (exp(1) - 1)* scale(i) ;
                    case 'noise' % Additive zero mean noise
                        noi = randn(1,final_index-init_index+1);
                        if ~isempty(opt_fau.noise_magnitude.snr)
                            noise_var = vari_sig * 10^(-opt_fau.noise_magnitude.snr/20);
                            fault = y(init_index:final_index) + noi * sqrt(noise_var);
                        elseif ~isempty(opt_fau.noise_magnitude.sigma_noise);
                            fault = y(init_index:final_index) + noi * sqrt(opt_fau.noise_magnitude.sigma_noise);    
                        end
                    case 'abr_mul' % Multplicative constant fault
                        fault = (y(init_index:final_index) - opt_output.ccoeff - mean(y))* (opt_fau.final_intensity_rate(i) + 1) + opt_output.ccoeff + mean(y);
                    case 'dri_mul' % Multplicative linearly increasing fault
                        fault = (y(init_index:final_index) - opt_output.ccoeff - mean(y)).* (1 + (1:n_faulty_sample) / opt_fau.per_fault_sample * final_intensity_rate(i) ) + opt_output.ccoeff + mean(y);
                    case 'exp_mul' % Multplicative exponentially increasing fault
                        fault = (y(init_index:final_index) - opt_output.ccoeff - mean(y)).* (1 +(exp(((0:(opt_fau.per_fault_sample-1))/(opt_fau.per_fault_sample-1))) - 1) / (exp(1) - 1) * opt_fau.final_intensity_rate(i) ) + opt_output.ccoeff + mean(y);
                    case 'stuck_at' %Stuck at fault
                        fault = repmat(y(init_index),1,final_index-init_index+1);
                    otherwise
                        fprintf('%s\n','The fault is not among those specified for this function');
                end
            elseif opt_fau.stru(i) == 1
            % Fault created modyfying the parameter vector
                opt_temp = opt_output;
                opt_temp.hor = opt_fau.per_fault_sample;
                if strcmp(opt_output.signal_type,'pendulum')
                    opt_temp.l = opt_output.l * (1 + opt_fau.final_intensity_rate(i));
                    opt_temp.k = opt_output.k;
                    opt_temp.m = opt_output.m;
                else
                    opt_temp.acoeff = opt_output.acoeff * (1 + opt_fau.final_intensity_rate(i));
                    opt_temp.bcoeff = opt_output.bcoeff * (1 + opt_fau.final_intensity_rate(i));
                    opt_temp.ccoeff = opt_output.ccoeff * (1 + opt_fau.final_intensity_rate(i));
                end
                x_cons = x(init_index:final_index);
                init_point = y(init_index - opt_output.na:init_index - 1);
                fault = signal_generation(x_cons,opt_temp,init_point);
            end
            yf(init_index:final_index) = fault;        
        end
    end
end

opt_fau.fau_times = opt_fau.fau_times + 1;
