function [best_model, orderO, cons_models, fitness] = idarmax(y, TrS_interval, modtype, ordersRange, do_akaike, VaS_interval)
% IDARMAX Find the best ar/arma model according to aic/fit trying all the orders defined by ordersRange
%   Input:
%    y - time series: y(:,1) = output; y(:,2) = input
%    TrS_interval - vector of size 2 specifying the first and the last samples of the training set
%    VaS_interval - vector of size 2 specifying the first and the last samples of the validation set
%    modtype - a string defining the model type (ar/arx/arma/armax)
%    ordersRange - structure containing the minimal and maximal orders to be analized 
%           oprdersRange.a - autoregressive range (required for ar/arx/arma/armax)
%           oprdersRange.b - exogenous range (required for arx/armax)
%           oprdersRange.c - moving average range (required for arma/armax)
%           oprdersRange.k - delay range (required for arx/arma/armax)
%    do_akaike - flag (0-1) for use of Akaike's information criterion (Default: aic = 1)
%
%   Output:
%    best_model - best model according to chosen fitness
%    order0 - order for the optimal model
%    cons_models - cell of the models considered for the model selection
%    fitness - fitness for the models considered in the model selection
%
% by eugene
%
% Revision history
% 2011 09 21  Giacomo Boracchi when VaS_interval = [] perform validation over the training
% 2013 05 06  Giacomo Boracchi BUG FIX Akaike
% 2013 11 12  Francesco Trovo check on y dimensions, on validation and training boundaries, modification of ordersRange definition
 
if ~exist('do_akaike','var')|| isempty(do_akaike)
    do_akaike = 1;
end

if ~exist('VaS_interval','var') || isempty(VaS_interval)
    VaS_interval = [];
end

% if akaike is performed then, no need of validation set
if (do_akaike == 1) && ~isempty(VaS_interval)
    TrS_interval = [TrS_interval(1), length(y)];
end

% Swap data matrix, rows correspond to different time instant
if size(y,2) > size(y,1);
    y = y';
end

%% Training and validation set construction
if size(y, 2) >= 2
    data_TrS = iddata(y(TrS_interval(1) : TrS_interval(2),1),y(TrS_interval(1) : TrS_interval(2),2));
else
    data_TrS = iddata(y(TrS_interval(1) : TrS_interval(2)));
end

if do_akaike == 0
    assert(VaS_interval(1) > TrS_interval(2));
    if size(y,2)>=2
        data_VaS = iddata(y(VaS_interval(1) : VaS_interval(2),1),y(VaS_interval(1) : VaS_interval(2),2));
        data_ToT = iddata(y(             1  : VaS_interval(2),1),y(             1  : VaS_interval(2),2));
    else
        data_VaS = iddata(y(VaS_interval(1) : VaS_interval(2)));
        data_ToT = iddata(y(             1  : VaS_interval(2),1));
    end
end


%% Model estimation
n_models = 0;
switch lower(modtype)
    case 'ar'
        check_field(ordersRange,'a');
        ordersRange = (ordersRange.a(1):ordersRange.a(2))';
        n_models = length(ordersRange);
        cons_models = cell(n_models,1);
        for ii = 1:n_models
            cons_models{ii} = ar(data_TrS, ii);
        end
    case 'arma'
        check_field(ordersRange,'a');
        check_field(ordersRange,'c');
        ordersRange =  struc(ordersRange.a(1) : ordersRange.a(2), ordersRange.c(1) : ordersRange.c(2),0:0);        
        n_models = size(ordersRange,1);
        cons_models = cell(n_models,1);
        for ii = 1:n_models
            % fit the model
            cons_models{ii} = armax(data_TrS,[ordersRange(ii,1), ordersRange(ii,2)]);
        end
    case 'arx'
        check_field(ordersRange,'a');
        check_field(ordersRange,'b');
        check_field(ordersRange,'k');
        ordersRange =  struc(ordersRange.a(1) : ordersRange.a(2), ordersRange.b(1) : ordersRange.b(2), ordersRange.k(1) : ordersRange.k(2));
        n_models = size(ordersRange,1);
        cons_models = cell(n_models,1);
        for ii = 1:n_models
            cons_models{ii} = arx(data_TrS,[ordersRange(ii,1), ordersRange(ii,2), ordersRange(ii,3)]);
        end
    case 'armax'
        check_field(ordersRange,'a');
        check_field(ordersRange,'c');
        check_field(ordersRange,'k');
        ordersRange = struc(ordersRange.a(1) : ordersRange.a(2), ordersRange.c(1) : ordersRange.c(2), ordersRange.k(1) : ordersRange.k(2));
        n_models = size(ordersRange,1);
        cons_models = cell(n_models,1);
        for ii = 1:n_models
            cons_models{ii} = armax(data_TrS, [ordersRange(ii, 1), ordersRange(ii, 2), ordersRange(ii, 3), 0]);
        end
end

%% Model selection
% fitness_tr = zeros(n_models,1);
% fitness_tr_corr = zeros(n_models,1);
fitness = zeros(n_models,1);

for ii = 1:n_models
    if do_akaike
        fitness(ii) = aic(cons_models);
        %p(ii) = cons_models{ii}.Report.Fit.LossFcn;
        %err_valid(ii) = log(cons_models{ii}.Report.Fit.LossFcn) + 2 * (sum(ordersRange(ii,[1 2]))) / TrS_interval(2);
        %fitness(ii) = log(cons_models{ii}.Report.Fit.LossFcn) + (sum(ordersRange(ii,[1 2]))) * log(TrS_interval(2));
    else
        ysim = predict(cons_models{ii}, data_ToT);
        resid_va = data_VaS.OutputData - ysim.y(VaS_interval(1):VaS_interval(2));
        resid_va = resid_va(abs(resid_va) > eps);
        fitness(ii) = sqrt(mse(resid_va));
    end
end

[~, ibst] = min(fitness);
orderO = ordersRange(ibst(1),:);
best_model = cons_models{ibst(1)};
end

function stru = check_field(stru,field)
    if isfield(stru,field)
        value = stru.(field);
        if length(value) < 2
            stru.(field) = [1 value];
        elseif length(stru.(field)) > 2
            error(['Too many value inserted in ' field ' field']);
        end
    else
        error(['Nonexisting required ' field ' field']);
    end
end
